import re
import os
import requests
from PIL import Image
#from SimpleX.utils import IsUrlValid
import urllib.parse


PURPLE =  '\033[35;40m'
BOLD_PURPLE = '\033[35;40;1m'
RED = '\033[31;40m'
BOLD_RED = '\033[31;40;1m'
RESET = '\033[m'



#### Checking Functions to validate that links are legit ####

def CheckUrl(url):
        """
        Checks if URL is actually reachable via Tor
        """
        proxies = {
                'http': 'socks5h://127.0.0.1:9050',
                'https': 'socks5h://127.0.0.1:9050'
        }
        try:
                status = requests.get(url,proxies=proxies, timeout=5).status_code
                if status != 502:
                        return True
                else:
                        return False
        except requests.ConnectionError as e:
                return False
        except requests.exceptions.ReadTimeout as e:
                return False


#### PROTECTIONS AGAINST MALICIOUS CSV INPUTS ####
def IsBannerValid(path: str) -> bool:
        """
        Checks if the banner.png file has the correct dimensions (240x60)
        """
        try:
                im = Image.open(path)
        except Exception as e:
                print("ERROR, EXCEPTION")
                return False
        width, height = im.size
        if width != 240 or height != 60:
                print("INVALID BANNER DIMENSIONS, HEIGHT=",height," WIDTH=",width)
                return False
        filesizeMB=os.path.getsize(path)/1024/1024
        if filesizeMB > 5:
                print("Banner filesize too large (>5Mb): ",os.path.getsize(path)/1024/1024,"MB")
                return False
        return True


def IsOnionValid(url: str)-> bool:
    """
    Checks if the domain(param) is a valid onion domain and return True else False.
    """
    try:
        pattern = re.compile("^[A-Za-z0-9.]+(.onion)?$")
        url = url.strip().removesuffix('/')
        if url.startswith('http://'):
            domain = url.split('/')[2]
            if pattern.fullmatch(domain) is not None:
                if len(domain.split('.')) > 3:
                    return False
                else:
                    if len(domain) < 62:
                        return False
                    return True
            elif pattern.fullmatch(domain) is None:
                return False
            else:
                return False
        else:
                        #TODO : edit the url to make sure it has http:// at the beginning, in case if it's missing? (problem is that it only returns true or false)
            if pattern.fullmatch(url) is not None:
                if len(url.split('.')) > 3:
                    return False
                else:
                    if len(url) < 62:
                        return False
                    return True
            elif pattern.fullmatch(url) is None:
                return False
            else:
                return False
    except Exception as e:
        return False

def IsSimpleXChatroomValid(url: str) -> bool:
    """Validate the SimpleX chatroom URL."""
    REQUIRED_SUBSTRING = "#/?v=2-7&smp=smp%3A%2F"

    # Step 1: Check if it starts with http://, https://, or simplex:/
    if url.startswith(('http://', 'https://', 'simplex:/')):
        # Step 1.5: If http:// or https://, check for valid clearnet or onion domain
        if url.startswith(('http://', 'https://')) and not IsUrlValid(url):
            return False
    elif not url.startswith('simplex:/'):
        return False  # Must start with one of the valid protocols

    # Step 2: Check for the presence of the required substring
    if REQUIRED_SUBSTRING not in url:
        return False  # Required substring not found

    # Step 3: Extract the part after "smp=smp%3A%2F"
    smp_start = url.find("smp=smp%3A%2F")
    if smp_start == -1:
        return False  # Required substring not found

    smp_start += len("smp=smp%3A%2F")
    smp_end = url.find("&", smp_start)
    if smp_end == -1:
        smp_end = len(url)  # Take until the end if no "&" is found

    smp_value = urllib.parse.unquote(url[smp_start:smp_end])  # Decode the URL-encoded string

    # Step 3.5: Check if the smp_value contains a valid hostname
    if '@' not in smp_value:
        return False  # Must contain '@' to separate fingerprint and hostname

    fingerprint, hostname = smp_value.split('@', 1)
    if not IsUrlValid(hostname):
        return False  # Invalid hostname

    # Step 4: Check for the presence of "%2F" in the original URL
    if "%2F" not in url:
        return False  # Required substring not found

    # If all checks pass, return True
    return True

def IsUrlValid(url:str)->bool:
        """
        Check if url is valid both dark net end clearnet.
        """
        pattern = re.compile(r"^[A-Za-z0-9:/._%-=#?&@]+$")
        onion_pattern = re.compile(r"^(\w+:)?(?://)?(\w+\.)?[a-z2-7]{56}\.onion")
        url = str(url)
        if len(url) < 4:
                return False
        if onion_pattern.match(url) is not None:
                return IsOnionValid(url)
        else:
                if not url.__contains__('.'):
                        return False
                if pattern.fullmatch(url) is None:
                        return False
                return True


#def IsUrlValid(url:str)->bool:
#        """
#        Check if url is valid both dark net end clearnet.
#        """
#        pattern  = re.compile("^[A-Za-z0-9:/.-]+$")
#        url = str(url)
#        if len(url) < 4:
#                return False
#        if url.endswith('.onion'):
#                return IsOnionValid(url)
#        else:
#                if not url.__contains__('.'):
#                        return False
#                if pattern.fullmatch(url) is None:
#                        return False
#                return True


def IsStatusValid(status: str)-> bool:
        """
        Checks if status contains only ['YES','NO']. Verbose only if False is returned
        """
        pattern = ['YES','NO','✔️','❌','']
        #pattern = ['YES','NO']
        status = str(status)
        status.strip()
        if (status not in pattern):
                return False

        return True


def IsScoreValid(score:str)->bool:
        """
        Check the Score is only "^[0-9.,]+$" with 8 max chars.
        """
        pattern = re.compile("^[0-9.,]+$")
        score = str(score)
        score.strip()
        if score in ['','nan']:
            return True
        if pattern.fullmatch(score) is None:
                return False
        elif len(score) > 8:
                return False
        return True


def IsDescriptionValid(desc:str)->bool:
        """
        Check the categories are only [a-zA-Z0-9.' ] with 256 max chars.
        """
        if desc == "":
                return True
        pattern = re.compile("^[A-Za-z0-9-.,' \"\(\)\/]+$")
        desc = str(desc)
        desc.strip()
        if pattern.fullmatch(desc) is None:
                return False
        if desc == "DEFAULT":
                return False
        elif len(desc) > 256:
                return False
        return True

def IsCategoryValid(categories: list)-> bool:
        """
        Check the categories are only [a-zA-Z0-9 ] with 64 max chars.
        """
        pattern = re.compile("^[A-Za-z0-9 ]+$")
        for category in categories:
                category.strip()
                if pattern.fullmatch(category) is None:
                        return False
                elif len(category) > 64:
                        return False
                else:
                       return True

def IsNameValid(name: str)->bool:
        """
        Check the parameter name only contains [a-zA-Z0-9 ] and is 64 chars long.
        """
        try:
                name = str(name)
        except Exception as e:
                return False
        pattern = re.compile("^[A-Za-z0-9 ]+$")
        name = name.strip()
        if (pattern.fullmatch(name) is None):
                return False
        elif len(name) > 64:
                return False
        return True


def print_colors(s:str=' ', bold:bool=False, is_error:bool = False, default:bool=False):
        """
        Helper function to print with colors
        """
        if is_error:
                print(f"{RED}{s}{RESET}")
        elif bold:
                print(f"{BOLD_PURPLE}{s}{RESET}")
        elif is_error and bold:
                print(f"{BOLD_RED}{s}{RESET}")
        elif default:
                print(f'{s}')
        else:
                print(f"{PURPLE}{s}{RESET}")
        
